import pygtk
pygtk.require('2.0')
import gtk, gobject

from sugar.activity import activityfactory
from sugar.activity.activity import Activity, ActivityToolbox, get_bundle_path

from gettext import gettext as _
import logging
import rsvg

import os
import sys
sys.path.append(os.path.join(os.environ['SUGAR_PATH'], 'services'))
try:
    from shell import bundleregistry
except:
    sys.path.append(os.path.join(os.environ['SUGAR_PATH'], 'shell'))
    try:
        from model import bundleregistry
    except:
        from sugar.activity import bundleregistry

BUBBLE_NAMES = (
    (_('Slider Puzzle'), "org.worldwideworkshop.olpc.SliderPuzzle"),
    (_('Poll Builder'), "org.worldwideworkshop.PollBuilder"),
    (_('Quiz Machine'), "org.worldwideworkshop.olpc.QuizMachine"),
    (_('Cartoon Builder'), "com.ywwg.CartoonBuilderActivity"),
    (_('BotBlox'), "org.worldwideworkshop.olpc.BotBlox"),
    (_('Flipsticks'), "org.worldwideworkshop.olpc.Flipsticks"),
    (_('My Bot Builder'), "org.worldwideworkshop.olpc.MyBotBuilder"),
    (_('Joke Machine'), "org.worldwideworkshop.olpc.JokeMachine"),
    (_('e-Card Builder'), "org.worldwideworkshop.olpc.ECardBuilder"),
    (_('Jigsaw Puzzle'), "org.worldwideworkshop.olpc.JigsawPuzzle"),
    (_('Story Builder'), "org.worldwideworkshop.olpc.storybuilder"),
    (_('Learning Center'), "org.worldwideworkshop.olpc.LearningCenter"),
    )

BUBBLE_POSITIONS = (
    (77, 525,  85),
    (47, 690, 187),
    (52, 580, 306),
    (67, 721, 301),
    (42, 637, 424),
    (62, 469, 384),
    (77, 296, 348),
    (63, 147, 385),
    (77,  13, 257),
    (63,  95, 123),
    (55, 250, 104),
    (50, 383, 135),
    )

BUBBLE_SET = [BUBBLE_NAMES[x]+BUBBLE_POSITIONS[x] for x in range(len(BUBBLE_POSITIONS))]

# TODO: this was stolen from bundle registry in jhbuild, because the XO doesn't have it yet. Remove eventually
def find_bundle(key):
    """Find a bundle in the registry"""
    breg = bundleregistry.get_registry()
    #return breg.find_bundle(key)
    key = key.lower()

    if hasattr(breg._bundles, 'values'):
        bundles = breg._bundles.values()
    else:
        bundles = breg._bundles
    for bundle in bundles:
        name = bundle.get_name().lower()
        service_name = bundle.get_service_name().lower()
        if name.find(key) != -1 or service_name.find(key) != -1:
            return bundle

    return None

class SensitiveImage (gtk.EventBox):
    __gsignals__ = {'bubble_enter' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_INT, gobject.TYPE_BOOLEAN)),
                    'bubble_leave' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_INT,)),
                    'bubble_click' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_INT,)),
                    }
    def __init__ (self):
        super(SensitiveImage, self).__init__()
        self.container = gtk.Fixed()
        self.container.show()
        self.add(self.container)
        bgimg = gtk.Image()
        bgimg.set_from_file("MaMaMenuPage.png")
        bgimg.show()
        self.container.put(bgimg, 0, 0)
        self.bubbles = []
        for n, b in enumerate(BUBBLE_SET):
            eb, i = self._prepare_circle(b[2])
            self.container.put(i, b[3], b[4])
            self.container.put(eb, b[3]+15, b[4]+15)
            self.bubbles.append((n,i,b[1] is not None and find_bundle(b[1]) is not None))
            eb.connect('enter-notify-event', self._enter_notify_cb, n, True)
            eb.connect('leave-notify-event', self._enter_notify_cb, n, False)
            eb.connect('button-release-event', self._click_notify_cb, n)
            i.hide()
        self.unlock()

    def lock (self):
        self.locked = True

    def unlock (self):
        self.locked = False
        # The lock_cache stores the first leave event and the last enter event, so we can set a consistent state on unlock
        if hasattr(self, 'lock_cache'):
            if self.lock_cache[0] is not None:
                self._enter_notify_cb(0,0,self.lock_cache[0], False)
            if self.lock_cache[1] is not None:
                self._enter_notify_cb(0,0,self.lock_cache[1], True)
        self.lock_cache = [None, None]

    def _prepare_circle (self, radius):
        circle_svg = file('icons/circle.svg.tmpl').read() % {'radius': str(radius),
                                                             'cx':str(radius+10),
                                                             'cy':str(radius+10)}
        handle = rsvg.Handle(data=circle_svg)
        i = gtk.Image()
        i.set_from_pixbuf(handle.get_pixbuf())
        eb = gtk.EventBox()
        eb.set_size_request(radius*2-10,radius*2-10)
        eb.set_visible_window(False)
        eb.show()
        return eb, i

    def _enter_notify_cb (self, w,evt,idx,go_in):
        if self.locked:
            if go_in:
                self.lock_cache[1] = idx
            else:
                if self.lock_cache[1] == None:
                    self.lock_cache[0] = idx
                else:
                    self.lock_cache[1] = None
            return
        if go_in:
            self.bubbles[idx][1].show()
            self.emit('bubble_enter', idx, self.bubbles[idx][2]) # last parameter states if we have found the bundle
        else:
            self.bubbles[idx][1].hide()
            self.emit('bubble_leave', idx)

    def _click_notify_cb (self, w,evt,idx):
        if self.locked:
            return
        self.emit('bubble_click', idx)
        
class BubbleLabel(gtk.Label):
    def __init__(self):
        super(BubbleLabel, self).__init__()

    def show(self, text):
        self.set_markup('<span font_desc="Sans 16">%s</span>' % text)

class MaMaMediaMenu (Activity):
    def __init__(self, handle):
        Activity.__init__(self, handle)
        self.prepare_activity(_("MaMaMedia Creative Center"))
        self.waiting = False
        table = gtk.Table(3,3,False)
        table.attach(gtk.Label(), 0,1,0,3)
        self.bubble_label = BubbleLabel()
        table.attach(self.bubble_label, 1,2,1,2)
        table.attach(gtk.Label(), 1,2,2,3)
        table.attach(gtk.Label(), 2,3,0,3)
        table.show_all()
        self.si = SensitiveImage()
        self.si.show()
        table.attach(self.si, 1,2,0,1,0,0)

        self.si.connect('bubble_enter', self._bubble_enter_cb)
        self.si.connect('bubble_leave', self._bubble_leave_cb)
        self.si.connect('bubble_click', self._bubble_click_cb)
        
        self._canvas.add(table)

    def prepare_activity (self, title):
        #TODO file locking
        propfile = os.path.expanduser("~/.sugar/default/org.worldwideworkshop.olpc.MMMPath")
        f = file(propfile, 'wb')
        f.write(str(get_bundle_path()))
        f.close()
        os.chdir(get_bundle_path())
        logging.debug('Starting %s activity... %s' % (title, str(get_bundle_path())))
        self.set_title(title)
        toolbox = ActivityToolbox(self)
        self.set_toolbox(toolbox)
        toolbox.show()
        if hasattr(self, '_jobject'):
            self._jobject.metadata['title'] = title
        title_widget = toolbox._activity_toolbar.title
        title_widget.set_size_request(title_widget.get_layout().get_pixel_size()[0] + 20, -1)
        self._canvas = gtk.EventBox()
        self._canvas.modify_bg(gtk.STATE_NORMAL,gtk.gdk.color_parse("#68CD05"))
        self._canvas.show()
        self.set_canvas(self._canvas)

    def _bubble_enter_cb (self, w, idx, exists):
        t = BUBBLE_SET[idx][0]
        if not exists:
            t = "%s (%s)" % (t, _("coming soon"))
        self.bubble_label.show(t)

    def _bubble_leave_cb (self, w, idx):
        self.bubble_label.show("")

    def _bubble_click_cb (self, w, idx):
        if BUBBLE_SET[idx][1] is not None:
            bundle = find_bundle(BUBBLE_SET[idx][1])
            if bundle is None:
                return
            handler = activityfactory.create(bundle.get_service_name())
            self._c1 = handler.connect("success", self._launch_success_cb, handler)
            self._c2 = handler.connect("error", self._launch_error_cb, handler, idx)
            self.bubble_label.show(_("Loading %s...") % BUBBLE_SET[idx][0])
            self.si.lock()
            c = gtk.gdk.Cursor(gtk.gdk.WATCH)
            self.window.set_cursor(c)

    def _launch_success_cb (self, exc, handler, *args):
        handler.disconnect(self._c1)
        handler.disconnect(self._c2)
        self.bubble_label.show("")
        self.si.unlock()
        self.window.set_cursor(None)

    def _launch_error_cb (self, exc, err, handler, idx, *args):
        handler.disconnect(self._c1)
        handler.disconnect(self._c2)
        self.bubble_label.show(_("error loading %s!") % BUBBLE_SET[idx][0])
        self.si.unlock()
        self.window.set_cursor(None)

